// Asynch_parse.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"

using namespace std;

#include "../../Kernel/kernelincludes.h"
#include "../../Multilevel/Blif.h"
#include "../../Multilevel/Node.h"
#include "../../Multilevel/Signal.h"
#include "../../Multilevel/Network.h"
#include "../../Kernel/orthogonalize.h"
#include "../../Kernel/Complement.h"

void Help()
{
	cout << "Syntax: Asynch_parse on.blif off.blif [options]" << endl;
	cout << "Options:" << endl;
	cout << "   -k n          k is the number of OR and AND gate inputs. Infinity is default." << endl;
	cout << "   -report       Report to stderr" << endl;
	cout << "                    source_file; source_inputs; source_outputs; source_nodes; source_non_trivial_nodes; source_andor_nodes; source_nontriv_inputs; source_terms; source_non_triv_terms; source_andor_terms; source_literals; source_GEs; source_levels; source_DCnodes; onset_terms; onset_nontriv_terms; onset_andor_terms; onset_literals; onset_GEs; offset_terms; offset_nontriv_terms; offset_andor_terms; offset_literals; offset_GEs; shared_terms; shared_GEs; effective_DC_nodes" << endl;
}

unsigned int CountInputs(Blif *blif)
{
	unsigned int cnt = 0;
	list<SoPLA>::iterator pla_it;

	for ( pla_it = blif->getData()->begin(); pla_it != blif->getData()->end(); pla_it++ )
		if ( (*pla_it).inputs() > 1 ) cnt += (*pla_it).inputs();
	return cnt;
}

unsigned int CountNonTrivTerms(Blif *blif)
{
	unsigned int cnt = 0;
	list<SoPLA>::iterator pla_it;

	for ( pla_it = blif->getData()->begin(); pla_it != blif->getData()->end(); pla_it++ )
		if ( (*pla_it).inputs() > 1 ) cnt += (*pla_it).getTerms();
	return cnt;
}

unsigned int CountTerms(Blif *blif)
{
	unsigned int cnt = 0;
	list<SoPLA>::iterator pla_it;

	for ( pla_it = blif->getData()->begin(); pla_it != blif->getData()->end(); pla_it++ )
		cnt += (*pla_it).getTerms();
	return cnt;
}

unsigned int CountANDORTerms(Blif *blif)
{
	unsigned int cnt = 0;
	list<SoPLA>::iterator pla_it;

	for ( pla_it = blif->getData()->begin(); pla_it != blif->getData()->end(); pla_it++ )
		if ( (*pla_it).inputs() > 1 && (*pla_it).getTerms() > 1 && (*pla_it).getTerms() != (*pla_it).getLiterals() ) cnt += (*pla_it).getTerms();
	return cnt;
}

unsigned int CountANDORNodes(Blif *blif)
{
	unsigned int cnt = 0;
	list<SoPLA>::iterator pla_it;

	for ( pla_it = blif->getData()->begin(); pla_it != blif->getData()->end(); pla_it++ )
		if ( (*pla_it).inputs() > 1 && (*pla_it).getTerms() > 1 && (*pla_it).getTerms() != (*pla_it).getLiterals() ) cnt++;
	return cnt;
}

unsigned int CountSharedTerms( PLA &pla )
{
	TermList::iterator ti;
	unsigned int count = 0;

	for ( ti = pla.getData()->begin(); ti != pla.getData()->end(); ti++ )
		if ( (*ti).getOM() == "11" ) count++;
	return count;
}

unsigned int CountShared( Blif &on_blif, Blif &off_blif, unsigned int &GEs, unsigned int &cnt )
// Computes shared terms and # of shared GEs
// Extracts terms (in terms of AND gates) from the BLIFs PLAs and compares them wrt. any permutation of variables
{
	list<SoPLA>::iterator pla_it1, pla_it2;
	CubeList::iterator ti1, ti2;
	unsigned int i, j;
	string inp1, inp2;
	vector<string> inames1, inames2;
	Blif tmpblif;
	bool equal;
	list<string> inames1_s, inames2_s;

	tmpblif = on_blif;
	tmpblif.AppendBlif(&off_blif);
	// tmpblif.SaveBlif("all.blif");
	
	for ( pla_it1 = tmpblif.getData()->begin(); pla_it1 != tmpblif.getData()->end(); ++pla_it1 )
		for ( ti1 = pla_it1->getData()->begin(); ti1 != pla_it1->getData()->end(); ++ti1 )
			ti1->tmp = 0;
	GEs = 0;
	cnt = 0;
	for ( pla_it1 = tmpblif.getData()->begin(); pla_it1 != tmpblif.getData()->end(); ++pla_it1 )
	  for ( ti1 = pla_it1->getData()->begin(); ti1 != pla_it1->getData()->end(); ++ti1 )
		  if ( ti1->tmp == 0 ) {
		  // extract 1st term
			inames1.clear();
			inp1.clear();
			if ( ti1->inputs() != 1 && ti1->getDim() != ti1->inputs()-1 ) {
				for ( i = 0; i < pla_it1->inputs(); i++ )
					if ( !ti1->isLit_dc(i) ) {
						inp1.push_back(ti1->getLit(i));
						inames1.push_back( pla_it1->getIName(i) );
					}
				pla_it2 = pla_it1;
				pla_it2++;
				while ( pla_it2 != tmpblif.getData()->end() ) {
					for ( ti2 = pla_it2->getData()->begin(); ti2 != pla_it2->getData()->end(); ++ti2 )
						if ( ti2->tmp == 0 ) {
							// extract 2nd term
							inames2.clear();
							inp2.clear();
							if ( ti2->inputs() != 1 && ti2->getDim() != ti2->inputs()-1 ) {
								for ( i = 0; i < pla_it2->inputs(); i++ )
									if ( !ti2->isLit_dc(i) ) {
										inp2.push_back(ti2->getLit(i));
										inames2.push_back( pla_it2->getIName(i) );
									}
							}
							// compare them
							equal = true;
							if ( inames1.size() != inames2.size() ) equal = false;
							else {
								inames1_s.clear();
								inames2_s.clear();
								for ( i = 0; i < inames1.size(); i++ ) {
									inames1_s.push_back(inames1[i]);
									inames2_s.push_back(inames2[i]);
								}
								inames1_s.sort();
								inames2_s.sort();
								if ( inames1_s != inames2_s ) equal = false;
								else for ( i = 0; i < inames1.size(); i++ )
									if ( equal ) 
										for ( j = 0; j < inames2.size(); j++ )
											if ( inames1[i] == inames2[j] && inp1[i] != inp2[j] ) {
												equal = false;
												break;
											}
							}
							if ( equal ) {
								cnt++;
								GEs += inp1.size();
								ti2->tmp = 1;
								break;
							}
						}
					pla_it2++;
				}
			}
		}
	return cnt;
}

int main(int argc, char* argv[])
{
    Blif on;
	Blif off;
	unsigned int i, j;
	unsigned int k = 0;
	char *onfile = NULL;
	char *offfile = NULL;
	bool report = false;
	list<SoPLA>::const_iterator cni;
	list<SoPLA>::iterator ni;
	list<Cube>::iterator ti;
	unsigned int ORs = 0;
	unsigned int ANDs = 0;
	unsigned int Cs = 0;

	unsigned int nontriv_nodes;
	unsigned int ON_GEs;
	unsigned int ON_AND_GEs;
	float ON_OR_GEs;
	unsigned int OFF_GEs;
	unsigned int OFF_AND_GEs;
	float OFF_OR_GEs;
	float CD_GEs;
	unsigned int shared_terms;
	unsigned int shared_GEs;
	unsigned int CD_levels;
	unsigned int ON_levels;
	unsigned int OFF_levels;


	cout << "Asynch_parse 1.0, (c) April 2011 by Petr Fiser" << endl;
	if ( argc < 3 ) {
        Help();
        exit(1);
    }

	try {
		i = 1;
		while ( i != argc ) {
			if ( argv[i][0] != '-' && onfile == NULL ) onfile = argv[i];
			else if ( argv[i][0] != '-' ) offfile = argv[i];
			else if ( strcmp( argv[i], "-k" ) == 0 ) {
				i++;
				k = atoi(argv[i]);
			}
			else if ( strcmp( argv[i], "-report" ) == 0 ) report = true;
			else {
				Help();
				exit(1);
			}
			i++;
		}
	} catch (...) {
        Help();
        exit(1);
    }

	try {
		on.LoadBlif(onfile);
	} catch (IOException e) {
		cerr << e.GetMsg() << endl;
		exit(1);
	} catch (bad_alloc e) {
		cerr << "Not enough memory" << endl;
		exit(1);
	}

	try {
		off.LoadBlif(offfile);
	} catch (IOException e) {
		cerr << e.GetMsg() << endl;
		exit(1);
	} catch (bad_alloc e) {
		cerr << "Not enough memory" << endl;
		exit(1);
	}

	if (report) cerr << "inputs; outputs; nodes; non_triv_nodes; AND_OR_nodes; ON_AND-OR_terms; ON_literals; ON_GEs; ON_AND_GEs; ON_OR_GEs; ON_levels; OFF_AND-OR_terms; OFF_literals; OFF_GEs; OFF_AND_GEs; OFF_OR_GEs; OFF_levels; shared_terms; shared_GEs; CD_GEs; CD_levels" << endl;

	try {
			// count inputs
		if ( on.inputs() != off.inputs() ) throw RuntimeException("The numbers of on- and off-set inputs differ");
		cout << "Inputs: " << on.inputs() << endl;
		if (report) cerr << on.inputs() << "; " << endl;

			// count outputs
		if ( on.outputs() != off.outputs() ) throw RuntimeException("The numbers of on- and off-set outputs differ");
		cout << "Outputs: " << on.outputs() << endl;
		if (report) cerr << on.outputs() << "; " << endl;

			// count nodes
		if ( on.GetNodes() != off.GetNodes() ) throw RuntimeException("The numbers of on- and off-set nodes differ");
		cout << "Nodes: " << on.GetNodes() << endl;
		if (report) cerr << on.GetNodes() << "; " << endl;

			// count non-trivial nodes
		i = 0;
		for ( cni = on.getConstDataRef().begin(); cni != on.getConstDataRef().end(); ++cni )
			if ( !cni->isTrivial() ) i++;
		j = 0;
		for ( cni = off.getConstDataRef().begin(); cni != off.getConstDataRef().end(); ++cni )
			if ( !cni->isTrivial() ) j++;
		if ( i != j ) throw RuntimeException("The numbers of non-trivial on- and off-set nodes differ");
		nontriv_nodes = i;
		cout << "Non-trivial nodes: " << i << endl;
		if (report) cerr << i << "; " << endl;

			// count AND-OR nodes
		i = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( ni->isAND_OR() ) i++;
		j = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( ni->isAND_OR() ) j++;
		if ( i != j ) throw RuntimeException("The numbers of AND-OR on- and off-set nodes differ");
		cout << "AND-OR nodes: " << i << endl;
		if (report) cerr << i << "; " << endl;

				// for on-set

			// count AND-OR nodes terms
		i = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( ni->isAND_OR() ) i += ni->getTerms();
		cout << "ON-set AND-OR nodes terms: " << i << endl;
		if (report) cerr << i << "; " << endl;

			// count literals (for non-trivial nodes)
		i = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) i += ni->getLiterals();
		cout << "ON-set literals (for non-trivial nodes): " << i << endl;
		if (report) cerr << i << "; " << endl;

			// count GEs (for non-trivial nodes)
		i = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) i += ni->getGEs();
		ON_GEs = i;
		cout << "ON-set GEs (for non-trivial nodes): " << (float)i/2 << endl;
		if (report) cerr << (float)i/2 << "; " << endl;

					// count AND GEs
		i = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) 
				for ( ti = ni->getData()->begin(); ti != ni->getData()->end(); ++ti ) {
					if ( k != 0 && ni->inputs()-ti->getDim() > k ) throw RuntimeException("ON-set node has more than k inputs");
					else if ( ni->inputs()-ti->getDim() > 1 ) i += ni->inputs()-ti->getDim();
				}
		ON_AND_GEs = i;
		cout << "ON-set AND GEs: " << (float)i/2 << endl;
		if (report) cerr << (float)i << "; " << endl;

					// count OR GEs
		ON_OR_GEs = 0;
		for ( ni = on.getData()->begin(); ni != on.getData()->end(); ++ni )
			if ( !ni->isTrivial() && ni->getTerms() > 1 ) 
				if ( k != 0) ON_OR_GEs += 0.5*(float)ni->getTerms()-1 + ceil(((float)ni->getTerms()-1)/((float)k-1));
				else ON_OR_GEs += 0.5*(float)ni->getTerms();

		cout << "ON-set OR GEs: " << ON_OR_GEs << endl;
		if (report) cerr << ON_OR_GEs << "; " << endl;

			// ON-set logic levels
		ON_levels = on.GetNAND_NANDLevels(k, true);
		cout << "ON-set logic levels: " << ON_levels << endl;
		if (report) cerr << ON_levels << "; " << endl;

				// for off-set

			// count AND-OR nodes terms
		i = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( ni->isAND_OR() ) i += ni->getTerms();
		cout << "OFF-set AND-OR nodes terms: " << i << endl;
		if (report) cerr << i << "; " << endl;

			// count literals (for non-trivial nodes)
		i = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) i += ni->getLiterals();
		cout << "OFF-set literals (for non-trivial nodes): " << i << endl;
		if (report) cerr << i << "; " << endl;

			// count GEs (for non-trivial nodes)
		i = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) i += ni->getGEs();
		OFF_GEs = i;
		cout << "OFF-set GEs (for non-trivial nodes): " << (float)i/2 << endl;
		if (report) cerr << (float)i/2 << "; " << endl;

			// count AND GEs
		i = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( !ni->isTrivial() ) 
				for ( ti = ni->getData()->begin(); ti != ni->getData()->end(); ++ti ) {
					if ( k != 0 && ni->inputs()-ti->getDim() > k ) throw RuntimeException("OFF-set node has more than k inputs");
					else if ( ni->inputs()-ti->getDim() > 1 ) i += ni->inputs()-ti->getDim();
				}
		OFF_AND_GEs = i;
		cout << "OFF-set AND GEs: " << (float)i/2 << endl;
		if (report) cerr << (float)i << "; " << endl;

							// count OR GEs
		OFF_OR_GEs = 0;
		for ( ni = off.getData()->begin(); ni != off.getData()->end(); ++ni )
			if ( !ni->isTrivial() && ni->getTerms() > 1 ) 
				if ( k != 0 ) OFF_OR_GEs += 0.5*(float)ni->getTerms()-1 + ceil(((float)ni->getTerms()-1)/((float)k-1));
				else OFF_OR_GEs += 0.5*(float)ni->getTerms();


		cout << "OFF-set OR GEs: " << OFF_OR_GEs << endl;
		if (report) cerr << OFF_OR_GEs << "; " << endl;

				// OFF-set logic levels
		OFF_levels = off.GetNAND_NANDLevels(k, true);
		cout << "OFF-set logic levels: " << OFF_levels << endl;
		if (report) cerr << OFF_levels << "; " << endl;


				// shared terms
		CountShared(on, off, shared_GEs, shared_terms);
		cout << "Shared terms: " << shared_terms << endl;
		cout << "Shared GEs: " << (float)shared_GEs/2 << endl;
		if (report) cerr << shared_terms << "; " << (float)shared_GEs/2 << "; " << endl;

				// CD logic GEs
		if ( k != 0 ) CD_GEs = 1.5*(float)(on.inputs() + nontriv_nodes) + ceil((float)(on.inputs() + nontriv_nodes - 1)/((float)k-1));
		else CD_GEs = 1.5*(float)(on.inputs() + nontriv_nodes) + 1;
		cout << "CD GEs: " << CD_GEs << endl;
		if (report) cerr << CD_GEs << "; " << endl;

				// CD levels
		if ( k != 0 ) CD_levels = ceil((float)(on.inputs() + nontriv_nodes - 1)/((float)k-1))+1;
		else CD_levels = 1;
		cout << "CD levels: " << CD_levels << endl;
		if (report) cerr << CD_levels << "; " << endl;

				// totals
		cout << "Total GEs: " << (float)(ON_AND_GEs + ON_OR_GEs*2 + OFF_AND_GEs + OFF_OR_GEs*2 - shared_GEs)/2 + CD_GEs << endl;

	} catch (RuntimeException e) {
		cerr << e.GetMsg() << endl;
		exit(1);
	}


	return 0;
}
